@@ -11,7 +11,7 @@ import re
 
 from mercurial import encoding, extensions
 from tortoisehg.util import hglib, version
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
