@@ -8,16 +8,18 @@
 import os
 import re
 import tempfile
+import time
 
 from mercurial import util, error, scmutil, phases
+from mercurial import obsolete  # delete if obsolete becomes enabled by default
 
-from tortoisehg.util import hglib, shlib, wconfig
+from tortoisehg.util import hglib, i18n, shlib, wconfig
+from tortoisehg.util.i18n import _
 
-from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt.messageentry import MessageEntry
 from tortoisehg.hgqt import cmdcore, cmdui, thgrepo
 from tortoisehg.hgqt import qtlib, qscilib, status, branchop, revpanel
-from tortoisehg.hgqt import hgrcutil, mqutil, lfprompt, i18n, partialcommit
+from tortoisehg.hgqt import hgrcutil, lfprompt
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -59,6 +61,60 @@ def commitopts2str(opts, mode='commit'):
                 optslist.append('--%s=%s' % (opt, value))
     return ' '.join(optslist)
 
+def mergecommitmessage(repo):
+    wctx = repo[None]
+    engmsg = repo.ui.configbool('tortoisehg', 'engmsg', False)
+    if wctx.p1().branch() == wctx.p2().branch():
+        msgset = i18n.keepgettext()._('Merge')
+        text = engmsg and msgset['id'] or msgset['str']
+        text = unicode(text)
+    else:
+        msgset = i18n.keepgettext()._('Merge with %s')
+        text = engmsg and msgset['id'] or msgset['str']
+        text = unicode(text) % hglib.tounicode(wctx.p2().branch())
+    return text
+
+def _getUserOptions(opts, *optionlist):
+    out = []
+    for opt in optionlist:
+        if opt not in opts:
+            continue
+        val = opts[opt]
+        if val is False:
+            continue
+        elif val is True:
+            out.append('--' + opt)
+        else:
+            out.append('--' + opt)
+            out.append(val)
+    return out
+
+def _mqNewRefreshCommand(repo, isnew, stwidget, pnwidget, message, opts, olist):
+    if isnew:
+        name = hglib.fromunicode(pnwidget.text())
+        if not name:
+            qtlib.ErrorMsgBox(_('Patch Name Required'),
+                              _('You must enter a patch name'))
+            pnwidget.setFocus()
+            return
+        cmdline = ['qnew', name]
+    else:
+        cmdline = ['qrefresh']
+    if message:
+        cmdline += ['--message=' + hglib.fromunicode(message)]
+    cmdline += _getUserOptions(opts, *olist)
+    files = ['--'] + [repo.wjoin(x) for x in stwidget.getChecked()]
+    addrem = [repo.wjoin(x) for x in stwidget.getChecked('!?')]
+    if len(files) > 1:
+        cmdline += files
+    else:
+        cmdline += ['--exclude', repo.root]
+    if addrem:
+        cmdlines = [['addremove'] + addrem, cmdline]
+    else:
+        cmdlines = [cmdline]
+    return cmdlines
+
 _topicmap = {
     'amend': _('Commit', 'start progress'),
     'commit': _('Commit', 'start progress'),
@@ -89,16 +145,15 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
         repoagent.configChanged.connect(self.refresh)
         repoagent.repositoryChanged.connect(self.repositoryChanged)
-        repoagent.workingBranchChanged.connect(self.refresh)
         self._repoagent = repoagent
         repo = repoagent.rawRepo()
         self._cmdsession = cmdcore.nullCmdSession()
         self._rev = rev
         self.lastAction = None
-        # Dictionary storing the last "commit messages"
+        # Dictionary storing the last (commit message, modified flag)
         # 'commit' is used for 'commit' and 'qnew', while
         # 'amend' is used for 'amend' and 'qrefresh'
-        self.lastCommitMsgs = {'commit': '', 'amend': ''}
+        self.lastCommitMsgs = {'commit': ('', False), 'amend': ('', False)}
         self.currentAction = None
 
         self.opts = opts = readopts(repo.ui) # user, date
@@ -140,15 +195,16 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
         self.recentMessagesButton = QToolButton(
             text=_('Copy message'),
-            popupMode=QToolButton.MenuButtonPopup,
+            popupMode=QToolButton.InstantPopup,
             toolTip=_('Copy one of the recent commit messages'))
-        self.recentMessagesButton.clicked.connect(
-            self.recentMessagesButton.showMenu)
+        m = QMenu(self.recentMessagesButton)
+        m.triggered.connect(self.msgSelected)
+        self.recentMessagesButton.setMenu(m)
         tbar.addWidget(self.recentMessagesButton)
         self.updateRecentMessages()
 
         tbar.addAction(_('Options')).triggered.connect(self.details)
-        tbar.setIconSize(QSize(16,16))
+        tbar.setIconSize(qtlib.smallIconSize())
 
         if _hasbugtraq and self.opts['bugtraqplugin'] != None:
             # We create the "Show Issues" button, but we delay its setup
@@ -196,7 +252,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             pnhbox = QHBoxLayout()
             self.pnlabel = QLabel()
             pnhbox.addWidget(self.pnlabel)
-            self.pnedit = mqutil.getPatchNameLineEdit()
+            self.pnedit = QLineEdit()
+            if hasattr(self.pnedit, 'setPlaceholderText'):  # Qt >= 4.7
+                self.pnedit.setPlaceholderText(_('### patch name ###'))
             self.pnedit.setMaximumWidth(250)
             pnhbox.addWidget(self.pnedit)
             pnhbox.addStretch()
@@ -294,7 +352,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             ctx = r.changectx('.')
             return (ctx.phase() != phases.public) \
                 and len(r.changectx(None).parents()) < 2 \
-                and not ctx.children()
+                and (obsolete._enabled or not ctx.children())
 
         acts = [
             ('commit', _('Commit changes'), _('Commit'), notpatch),
@@ -380,7 +438,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.pnlabel.setVisible(True)
             self.pnedit.setVisible(True)
             self.pnedit.setFocus()
-            self.pnedit.setText(mqutil.defaultNewPatchName(self.repo))
+            pn = time.strftime('%Y-%m-%d_%H-%M-%S')
+            pn += '_r%d+.diff' % self.repo['.'].rev()
+            self.pnedit.setText(pn)
             self.pnedit.selectAll()
             self.stwidget.setPatchContext(None)
             refreshwctx = refresh and oldpctx is not None
@@ -407,16 +467,20 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 allowcs = len(self.repo.parents()) == 1
         if curraction._name in ('qref', 'amend'):
             if self.lastAction not in ('qref', 'amend'):
-                self.lastCommitMsgs['commit'] = self.msgte.text()
-            if self.lastCommitMsgs['amend']:
-                msg = self.lastCommitMsgs['amend']
-            else:
-                msg = hglib.tounicode(pctx.description())
-            self.setMessage(msg)
+                self.lastCommitMsgs['commit'] = (self.msgte.text(),
+                                                 self.msgte.isModified())
+            if self.lastCommitMsgs['amend'][0]:
+                self.setMessage(*self.lastCommitMsgs['amend'])
+            elif oldpctx is None or oldpctx.node() != pctx.node():
+                # pctx must be refreshed if hash changes
+                self.setMessage(hglib.tounicode(pctx.description()))
         else:
             if self.lastAction in ('qref', 'amend'):
-                self.lastCommitMsgs['amend'] = self.msgte.text()
-                self.setMessage(self.lastCommitMsgs['commit'])
+                self.lastCommitMsgs['amend'] = (self.msgte.text(),
+                                                self.msgte.isModified())
+                self.setMessage(*self.lastCommitMsgs['commit'])
+            elif len(self.repo[None].parents()) > 1:
+                self.setMessage(mergecommitmessage(self.repo))
         if curraction._name == 'amend':
             self.stwidget.defcheck = 'amend'
         else:
@@ -445,7 +509,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         commandlines = []
         newbranch = False
         branch = hglib.fromunicode(self.branchop)
-        if branch in repo.branchtags():
+        if branch in repo.branchmap():
             # response: 0=Yes, 1=No, 2=Cancel
             if branch in [p.branch() for p in repo.parents()]:
                 resp = 0
@@ -491,11 +555,11 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             wholecmdlines.extend(cmdlines)
 
         olist = ('user', 'date')
-        cmdlines = mqutil.mqNewRefreshCommand(self.repo,
-                                              curraction._name == 'qnew',
-                                              self.stwidget, self.pnedit,
-                                              self.msgte.text(), self.opts,
-                                              olist)
+        cmdlines = _mqNewRefreshCommand(self.repo,
+                                        curraction._name == 'qnew',
+                                        self.stwidget, self.pnedit,
+                                        self.msgte.text(), self.opts,
+                                        olist)
         if not cmdlines:
             return
         wholecmdlines.extend(cmdlines)
@@ -506,7 +570,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         'Status widget is displaying a new file'
         if not (wfile and contents):
             return
-        if self.msgte.autoCompletionThreshold() < 0:
+        if self.msgte.autoCompletionThreshold() <= 0:
             # do not search for tokens if auto completion is disabled
             # pygments has several infinite loop problems we'd like to avoid
             return
@@ -523,6 +587,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         tokens.add(wfile)
         tokens.add(os.path.basename(wfile))
         try:
+            from mercurial import demandimport; demandimport.disable()
             from pygments.lexers import guess_lexer_for_filename
             from pygments.token import Token
             from pygments.util import ClassNotFound
@@ -571,7 +636,8 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         mode = 'commit'
         if len(self.repo.parents()) > 1:
             mode = 'merge'
-        dlg = DetailsDialog(self.opts, self.userhist, self, mode=mode)
+        dlg = DetailsDialog(self._repoagent, self.opts, self.userhist, self,
+                            mode=mode)
         dlg.finished.connect(dlg.deleteLater)
         dlg.setWindowFlags(Qt.Sheet)
         dlg.setWindowModality(Qt.WindowModal)
@@ -579,17 +645,29 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.opts.update(dlg.outopts)
             self.refresh()
 
-    @pyqtSlot()
-    def repositoryChanged(self):
-        'Repository has detected a changelog / dirstate change'
+    @pyqtSlot(int)
+    def repositoryChanged(self, flags):
+        if flags & thgrepo.WorkingParentChanged:
+            self._refreshWorkingState()
+        elif flags & thgrepo.WorkingBranchChanged:
+            self.refresh()
+
+    def _refreshWorkingState(self):
+        curraction = self.mqgroup.checkedAction()
+        if curraction._name == 'commit' and not self.msgte.isModified():
+            # default merge or close-branch message is outdated if new commit
+            # was made by other widget or process
+            self.msgte.clear()
+        self.lastCommitMsgs['amend'] = ('', False)  # avoid loading stale cache
+        # refresh() may load/save the stale 'amend' message in commitSetAction()
+        self.refresh()
+        self.stwidget.refreshWctx() # Trigger reload of working context
         # clear the last 'amend' message
         # do not clear the last 'commit' message because there are many cases
         # in which we may write a commit message first, modify the repository
         # (e.g. amend or update and merge uncommitted changes) and then do the
         # actual commit
-        self.lastCommitMsgs['amend'] = ''
-        self.refresh()
-        self.stwidget.refreshWctx() # Trigger reload of working context
+        self.lastCommitMsgs['amend'] = ('', False)  # clear saved stale cache
 
     @pyqtSlot()
     def refreshWctx(self):
@@ -650,7 +728,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.commitSetAction()
 
     def branchOp(self):
-        d = branchop.BranchOpDialog(self.repo, self.branchop, self)
+        d = branchop.BranchOpDialog(self._repoagent, self.branchop, self)
         d.setWindowFlags(Qt.Sheet)
         d.setWindowModality(Qt.WindowModal)
         if d.exec_() == QDialog.Accepted:
@@ -667,14 +745,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     def canUndo(self):
         'Returns undo description or None if not valid'
-        if os.path.exists(self.repo.sjoin('undo')):
-            try:
-                args = self.repo.opener('undo.desc', 'r').read().splitlines()
-                if args[1] != 'commit':
-                    return None
-                return _('Rollback commit to revision %d') % (int(args[0]) - 1)
-            except (IOError, IndexError, ValueError):
-                pass
+        desc, oldlen = hglib.readundodesc(self.repo)
+        if desc == 'commit':
+            return _('Rollback commit to revision %d') % (oldlen - 1)
         return None
 
     def rollback(self):
@@ -689,12 +762,12 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     def updateRecentMessages(self):
         # Define a menu that lists recent messages
-        m = QMenu(self.recentMessagesButton)
+        m = self.recentMessagesButton.menu()
+        m.clear()
         for s in self.msghistory:
             title = s.split('\n', 1)[0][:70]
-            def overwriteMsg(newMsg): return lambda: self.msgSelected(newMsg)
-            m.addAction(title).triggered.connect(overwriteMsg(s))
-        self.recentMessagesButton.setMenu(m)
+            a = m.addAction(title)
+            a.setData(s)
 
     def getMessage(self, allowreplace):
         text = self.msgte.text()
@@ -706,20 +779,22 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             else:
                 raise
 
-    def msgSelected(self, message):
+    @pyqtSlot(QAction)
+    def msgSelected(self, action):
         if self.msgte.text() and self.msgte.isModified():
             d = QMessageBox.question(self, _('Confirm Discard Message'),
                         _('Discard current commit message?'),
                         QMessageBox.Ok | QMessageBox.Cancel)
             if d != QMessageBox.Ok:
                 return
+        message = action.data().toString()
         self.setMessage(message)
         self.msgte.setFocus()
 
-    def setMessage(self, msg):
+    def setMessage(self, msg, modified=False):
         self.msgte.setText(msg)
         self.msgte.moveCursorToEnd()
-        self.msgte.setModified(False)
+        self.msgte.setModified(modified)
 
     def canExit(self):
         if not self.stwidget.canExit():
@@ -728,7 +803,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     def loadSettings(self, s, prefix):
         'Load history, etc, from QSettings instance'
-        repoid = str(self.repo[0])
+        repoid = hglib.shortrepoid(self.repo)
         lpref = prefix + '/commit/' # local settings (splitter, etc)
         gpref = 'commit/'           # global settings (history, etc)
         # message history is stored in unicode
@@ -755,7 +830,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
     def saveSettings(self, s, prefix):
         'Save history, etc, in QSettings instance'
         try:
-            repoid = str(self.repo[0])
+            repoid = hglib.shortrepoid(self.repo)
             lpref = prefix + '/commit/'
             gpref = 'commit/'
             s.setValue(lpref+'split', self.split.saveState())
@@ -802,7 +877,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             return
 
         if not msg:
-            qtlib.WarningMsgBox(_('Nothing Commited'),
+            qtlib.WarningMsgBox(_('Nothing Committed'),
                                 _('Please enter commit message'),
                                 parent=self)
             self.msgte.setFocus()
@@ -811,11 +886,17 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         linkmandatory = self.repo.ui.configbool('tortoisehg',
                                                 'issue.linkmandatory', False)
         if linkmandatory:
-            issueregex = self.repo.ui.config('tortoisehg', 'issue.regex')
+            issueregex = None
+            s = self.repo.ui.config('tortoisehg', 'issue.regex')
+            if s:
+                try:
+                    issueregex = re.compile(s)
+                except re.error:
+                    pass
             if issueregex:
-                m = re.search(issueregex, msg)
+                m = issueregex.search(msg)
                 if not m:
-                    qtlib.WarningMsgBox(_('Nothing Commited'),
+                    qtlib.WarningMsgBox(_('Nothing Committed'),
                                         _('No issue link was found in the '
                                           'commit message.  The commit message '
                                           'should contain an issue link.  '
@@ -857,12 +938,26 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.stwidget.tv.setFocus()
             return
 
-        user = qtlib.getCurrentUsername(self, self.repo, self.opts)
-        if not user:
-            return
-        self.addUsernameToHistory(user)
+        # username will be prompted as necessary by hg if ui.askusername
+        user = self.opts.get('user')
+        if not amend and not repo.ui.configbool('ui', 'askusername'):
+            # TODO: no need to specify --user if it was read from ui
+            user = qtlib.getCurrentUsername(self, self.repo, self.opts)
+            if not user:
+                return
+            self.addUsernameToHistory(user)
 
         checkedUnknowns = self.stwidget.getChecked('?I')
+        if checkedUnknowns and 'largefiles' in repo.extensions():
+            result = lfprompt.promptForLfiles(self, repo.ui, repo,
+                                              checkedUnknowns)
+            if not result:
+                return
+            checkedUnknowns, lfiles = result
+            if lfiles:
+                cmd = ['add', '--large', '--']
+                cmd.extend(map(hglib.escapepath, lfiles))
+                commandlines.append(cmd)
         if checkedUnknowns:
             confirm = self.repo.ui.configbool('tortoisehg',
                                               'confirmaddfiles', True)
@@ -875,18 +970,8 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             else:
                 res = 0
             if res == 0:
-                haslf = 'largefiles' in repo.extensions()
-                if haslf:
-                    result = lfprompt.promptForLfiles(self, repo.ui, repo,
-                                                      checkedUnknowns)
-                    if not result:
-                        return
-                    checkedUnknowns, lfiles = result
-                    if lfiles:
-                        cmd = ['add', '--large'] + \
-                            [repo.wjoin(f) for f in lfiles]
-                        commandlines.append(cmd)
-                cmd = ['add'] + [repo.wjoin(f) for f in checkedUnknowns]
+                cmd = ['add', '--']
+                cmd.extend(map(hglib.escapepath, checkedUnknowns))
                 commandlines.append(cmd)
             else:
                 return
@@ -903,31 +988,20 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             else:
                 res = 0
             if res == 0:
-                cmd = ['remove'] + [repo.wjoin(f) for f in checkedMissing]
+                cmd = ['remove', '--']
+                cmd.extend(map(hglib.escapepath, checkedMissing))
                 commandlines.append(cmd)
             else:
                 return
-        try:
-            date = self.opts.get('date')
-            if date:
-                util.parsedate(date)
-                dcmd = ['--date', date]
-            else:
-                dcmd = []
-        except error.Abort, e:
-            if e.hint:
-                err = _('%s (hint: %s)') % (hglib.tounicode(str(e)),
-                                            hglib.tounicode(e.hint))
-            else:
-                err = hglib.tounicode(str(e))
-            self.showMessage.emit(err)
-            dcmd = []
-        cmdline = ['commit', '--verbose', '--user', user, '--message='+msg]
-        cmdline += dcmd + brcmd
+        cmdline = ['commit', '--verbose', '--message='+msg]
+        if user:
+            cmdline.extend(['--user', user])
+        date = self.opts.get('date')
+        if date:
+            cmdline += ['--date', date]
+        cmdline += brcmd
 
         if partials:
-            partialcommit.uisetup(repo.ui)
-
             # write patch for partial change selections to temp file
             fd, tmpname = tempfile.mkstemp(prefix='thg-patch-')
             fp = os.fdopen(fd, 'wb')
@@ -955,12 +1029,12 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             assert not self.stwidget.partials
 
         cmdline.append('--')
-        cmdline.extend([repo.wjoin(f) for f in self.files])
+        cmdline.extend(map(hglib.escapepath, self.files))
         if len(repo.parents()) == 1:
             for fname in self.opts.get('autoinc', '').split(','):
                 fname = fname.strip()
                 if fname:
-                    cmdline.append(repo.wjoin(fname))
+                    cmdline.append('glob:%s' % fname)
         commandlines.append(cmdline)
 
         if self.opts.get('pushafter'):
@@ -995,7 +1069,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.branchop = None
             umsg = self.msgte.text()
             if self.currentAction not in ('qref', 'amend'):
-                self.lastCommitMsgs['commit'] = ''
+                self.lastCommitMsgs['commit'] = ('', False)
                 if self.currentAction == 'commit':
                     # capture last message for BugTraq plugin
                     self.lastmessage = self.getMessage(True)
@@ -1005,16 +1079,20 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 if self.currentAction == 'commit':
                     shlib.shell_notify(self.files)
                     self.commitComplete.emit()
+        elif ret == 1 and self.currentAction in ('amend', 'commit'):
+            qtlib.WarningMsgBox(_('Nothing Committed'),
+                                _('Nothing changed.'),
+                                parent=self)
         else:
             cmdui.errorMessageBox(self._cmdsession, self,
                                   _('Commit', 'window title'))
 
 class DetailsDialog(QDialog):
     'Utility dialog for configuring uncommon settings'
-    def __init__(self, opts, userhistory, parent, mode='commit'):
+    def __init__(self, repoagent, opts, userhistory, parent, mode='commit'):
         QDialog.__init__(self, parent)
-        self.setWindowTitle(_('%s - commit options') % parent.repo.displayname)
-        self.repo = parent.repo
+        self.setWindowTitle(_('%s - commit options') % repoagent.displayName())
+        self._repoagent = repoagent
 
         layout = QVBoxLayout()
         self.setLayout(layout)
@@ -1157,6 +1235,10 @@ class DetailsDialog(QDialog):
         self.bb = bb
         layout.addWidget(bb)
 
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
+
     def saveInRepo(self):
         fn = os.path.join(self.repo.root, '.hg', 'hgrc')
         self.saveToPath([fn])
@@ -1318,6 +1400,7 @@ class CommitDialog(QDialog):
         QDialog.__init__(self, parent)
         self.setWindowFlags(Qt.Window)
         self.setWindowIcon(qtlib.geticon('hg-commit'))
+        self._repoagent = repoagent
         self.pats = pats
         self.opts = opts
 
@@ -1361,8 +1444,7 @@ class CommitDialog(QDialog):
         repoagent.repositoryChanged.connect(self.updateUndo)
         commit.commitComplete.connect(self.postcommit)
 
-        repo = repoagent.rawRepo()
-        self.setWindowTitle(_('%s - commit') % repo.displayname)
+        self.setWindowTitle(_('%s - commit') % repoagent.displayName())
         self.commit = commit
         self.commit.reload()
         self.updateUndo()
@@ -1375,9 +1457,7 @@ class CommitDialog(QDialog):
             self._subdialogs.open(link[len('repo:'):])
 
     def _createSubDialog(self, uroot):
-        # TODO: do not instantiate repo here
-        repo = thgrepo.repository(None, hglib.fromunicode(uroot))
-        repoagent = repo._pyqtobj
+        repoagent = self._repoagent.subRepoAgent(uroot)
         return CommitDialog(repoagent, [], {}, parent=self)
 
     @pyqtSlot()
