@@ -11,7 +11,7 @@ from PyQt4.QtGui import *
 import os
 
 from tortoisehg.util import hglib
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import qtlib, cmdcore, cmdui, resolve, thgrepo, wctxcleaner
 from tortoisehg.hgqt import csinfo, cslist
 
@@ -174,7 +174,7 @@ class GraftDialog(QDialog):
 
     def graft(self):
         self.graftbtn.setEnabled(False)
-        self.cancelbtn.setShown(False)
+        self.cancelbtn.setVisible(False)
         opts = dict((n, w.isChecked()) for n, w in self._optchks.iteritems())
         itool = opts.pop('autoresolve') and 'merge' or 'fail'
         opts['config'] = 'ui.merge=internal:%s' % itool
